"use strict";

var argx = require('argx'),
    path = require('path'),
    fs = require('fs'),
    findout = require('findout'),
    types = require('./types'),
    async = require('async'),
    listScaffolds = require('./scaffolding/list_scaffolds'),
    askScaffold = require('./scaffolding/ask_scaffold'),
    applyScaffold = require('./scaffolding/apply_scaffold'),
    _skipError = require('./_skip_error');

/** @lends apemanScaffold */
function apemanScaffold(type, dest, options, callback) {
    var args = argx(arguments);
    callback = args.pop('function') || argx.noop;
    type = args.shift('string');
    options = args.pop('object') || {};
    dest = args.shift('string');

    var listing = !type && !dest;
    if (listing) {
        listScaffolds(types, callback);
        return;
    }


    if (!dest) {
        callback(new Error('dest is required.'));
        return;
    }
    if (typeof(type) === 'string') {
        if (types[type]) {
            type = types[type]['module'];
        } else {
            type = findout(type);
        }
    }
    if (!type) {
        callback(new Error('type not found.'));
    }

    fs.exists(dest, function (exists) {
        var skip = exists && !options.force;
        if (skip) {
            callback(_skipError(dest));
            return;
        }
        async.waterfall([
            function (callback) {
                askScaffold(type, dest, {
                    straight: options.straight
                }, callback);
            },
            function (asked, callback) {
                applyScaffold(type, dest, asked, {
                    silent: options.silent
                }, callback);

            }
        ], callback);
    });
}


module.exports = apemanScaffold;

