/**
 * Logger for list.
 * @constructor ListLogger
 * @returns A logger instance.
 */

"use strict";

var colorprint = require('colorprint'),
    clc = require('cli-color');

/** @lends ListLogger */
module.exports = colorprint.define(
    /** @lends ListLogger.prototype */
    {
        /**
         * Log task list.
         * @param {object[]} types - Types to show.
         */
        logTypes: function (types) {
            var s = this;
            s.debug('Available types:');
            Object.keys(types).forEach(function (name) {
                var desc = s._descType(types[name]);
                var space = ' ';
                while (name.length + space.length < 20) {
                    space += ' ';
                }
                s.debug("  %s%s", clc.black.bold(name), clc.blackBright([space, '//', desc].join(' ')));
            });
            s.debug('');
        },
        _descType: function (type) {
            var pkg = type.pkg || {};
            return pkg.description;
        }
    }
);
